/*
 * Copyright 2010-2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.amazonaws.services.rekognition.model.transform;

import static com.amazonaws.util.StringUtils.UTF8;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;

import android.text.TextUtils;

import com.amazonaws.AmazonClientException;
import com.amazonaws.Request;
import com.amazonaws.DefaultRequest;
import com.amazonaws.http.HttpMethodName;
import com.amazonaws.services.rekognition.model.*;
import com.amazonaws.transform.Marshaller;
import com.amazonaws.util.BinaryUtils;
import com.amazonaws.util.DateUtils;
import com.amazonaws.util.StringUtils;
import com.amazonaws.util.StringInputStream;
import com.amazonaws.util.json.AwsJsonWriter;
import com.amazonaws.util.json.JsonUtils;

/**
 * JSON request marshaller for CreateStreamProcessorRequest
 */
public class CreateStreamProcessorRequestMarshaller implements
        Marshaller<Request<CreateStreamProcessorRequest>, CreateStreamProcessorRequest> {

    public Request<CreateStreamProcessorRequest> marshall(
            CreateStreamProcessorRequest createStreamProcessorRequest) {
        if (createStreamProcessorRequest == null) {
            throw new AmazonClientException(
                    "Invalid argument passed to marshall(CreateStreamProcessorRequest)");
        }

        Request<CreateStreamProcessorRequest> request = new DefaultRequest<CreateStreamProcessorRequest>(
                createStreamProcessorRequest, "AmazonRekognition");
        String target = "RekognitionService.CreateStreamProcessor";
        request.addHeader("X-Amz-Target", target);
        request.setHttpMethod(HttpMethodName.POST);

        String uriResourcePath = "/";
        request.setResourcePath(uriResourcePath);
        try {
            StringWriter stringWriter = new StringWriter();
            AwsJsonWriter jsonWriter = JsonUtils.getJsonWriter(stringWriter);
            jsonWriter.beginObject();

            if (createStreamProcessorRequest.getInput() != null) {
                StreamProcessorInput input = createStreamProcessorRequest.getInput();
                jsonWriter.name("Input");
                StreamProcessorInputJsonMarshaller.getInstance().marshall(input, jsonWriter);
            }
            if (createStreamProcessorRequest.getOutput() != null) {
                StreamProcessorOutput output = createStreamProcessorRequest.getOutput();
                jsonWriter.name("Output");
                StreamProcessorOutputJsonMarshaller.getInstance().marshall(output, jsonWriter);
            }
            if (createStreamProcessorRequest.getName() != null) {
                String name = createStreamProcessorRequest.getName();
                jsonWriter.name("Name");
                jsonWriter.value(name);
            }
            if (createStreamProcessorRequest.getSettings() != null) {
                StreamProcessorSettings settings = createStreamProcessorRequest.getSettings();
                jsonWriter.name("Settings");
                StreamProcessorSettingsJsonMarshaller.getInstance().marshall(settings, jsonWriter);
            }
            if (createStreamProcessorRequest.getRoleArn() != null) {
                String roleArn = createStreamProcessorRequest.getRoleArn();
                jsonWriter.name("RoleArn");
                jsonWriter.value(roleArn);
            }
            if (createStreamProcessorRequest.getTags() != null) {
                java.util.Map<String, String> tags = createStreamProcessorRequest.getTags();
                jsonWriter.name("Tags");
                jsonWriter.beginObject();
                for (java.util.Map.Entry<String, String> tagsEntry : tags.entrySet()) {
                    String tagsValue = tagsEntry.getValue();
                    if (tagsValue != null) {
                        jsonWriter.name(tagsEntry.getKey());
                        jsonWriter.value(tagsValue);
                    }
                }
                jsonWriter.endObject();
            }
            if (createStreamProcessorRequest.getNotificationChannel() != null) {
                StreamProcessorNotificationChannel notificationChannel = createStreamProcessorRequest
                        .getNotificationChannel();
                jsonWriter.name("NotificationChannel");
                StreamProcessorNotificationChannelJsonMarshaller.getInstance().marshall(
                        notificationChannel, jsonWriter);
            }
            if (createStreamProcessorRequest.getKmsKeyId() != null) {
                String kmsKeyId = createStreamProcessorRequest.getKmsKeyId();
                jsonWriter.name("KmsKeyId");
                jsonWriter.value(kmsKeyId);
            }
            if (createStreamProcessorRequest.getRegionsOfInterest() != null) {
                java.util.List<RegionOfInterest> regionsOfInterest = createStreamProcessorRequest
                        .getRegionsOfInterest();
                jsonWriter.name("RegionsOfInterest");
                jsonWriter.beginArray();
                for (RegionOfInterest regionsOfInterestItem : regionsOfInterest) {
                    if (regionsOfInterestItem != null) {
                        RegionOfInterestJsonMarshaller.getInstance().marshall(
                                regionsOfInterestItem, jsonWriter);
                    }
                }
                jsonWriter.endArray();
            }
            if (createStreamProcessorRequest.getDataSharingPreference() != null) {
                StreamProcessorDataSharingPreference dataSharingPreference = createStreamProcessorRequest
                        .getDataSharingPreference();
                jsonWriter.name("DataSharingPreference");
                StreamProcessorDataSharingPreferenceJsonMarshaller.getInstance().marshall(
                        dataSharingPreference, jsonWriter);
            }

            jsonWriter.endObject();
            jsonWriter.close();
            String snippet = stringWriter.toString();
            byte[] content = snippet.getBytes(UTF8);
            request.setContent(new StringInputStream(snippet));
            request.addHeader("Content-Length", Integer.toString(content.length));
        } catch (Throwable t) {
            throw new AmazonClientException(
                    "Unable to marshall request to JSON: " + t.getMessage(), t);
        }
        if (!request.getHeaders().containsKey("Content-Type")) {
            request.addHeader("Content-Type", "application/x-amz-json-1.1");
        }

        return request;
    }
}
